package com.hz.pm.api.common.statemachine.action;

import cn.hutool.core.lang.Assert;
import com.hz.pm.api.common.model.constant.StateMachineConst;
import com.hz.pm.api.common.statemachine.builder.impl.ProjectStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

/**
 * 项目申报状态机action集合类
 *
 * @author CMM
 * @since 2023/02/08 13:48
 */
@Slf4j
@WithStateMachine(id = ProjectStateMachineBuilderImpl.MACHINE_ID)
public class ProjectStateChangeAction {

    private static final String PROJECT_DECLARE = StateMachineConst.PROJECT_DECLARE;

    private Project getProject(Message<ProjectStateChangeEvent> message) {
        Project project = (Project) message.getHeaders().get(PROJECT_DECLARE);
        Assert.notNull(project, "未获取到需要状态变更的项目信息");
        return project;
    }

    @OnTransition(source = "UNDER_INTERNAL_AUDIT", target = "PENDING_PREQUALIFICATION")
    public void UNDER_INTERNAL_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode());
    }

    @OnTransition(source = "UNDER_INTERNAL_AUDIT", target = "UNDER_INTERNAL_AUDIT_NOT_PASS")
    public void UNDER_INTERNAL_REJECT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.UNDER_INTERNAL_AUDIT_NOT_PASS.getCode());
    }

    @OnTransition(source = "UNDER_INTERNAL_AUDIT", target = "TO_BE_DECLARED")
    public void UNDER_INTERNAL_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.TO_BE_DECLARED.getCode());
    }

    @OnTransition(source = "UNDER_INTERNAL_AUDIT_NOT_PASS", target = "UNDER_INTERNAL_AUDIT")
    public void UNDER_INTERNAL_REJECT_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.UNDER_INTERNAL_AUDIT.getCode());
    }


    @OnTransition(source = "PENDING_PREQUALIFICATION", target = "PENDING_PREQUALIFICATION_CHOICE")
    public void PRELIMINARY_REVIEW_DECLARE(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PENDING_PREQUALIFICATION_CHOICE.getCode());
    }

    @OnTransition(source = "PENDING_PREQUALIFICATION", target = "UNDER_INTERNAL_AUDIT")
    public void PENDING_PREQUALIFICATION_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.UNDER_INTERNAL_AUDIT.getCode());
    }


    @OnTransition(source = "JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS", target = "JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_SUCCESS")
    public void PROVINCIAL_DEPARTMENT_REVIEW_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_SUCCESS.getCode());
    }

    @OnTransition(source = "JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_SUCCESS", target = "PRE_APPLYING")
    public void PROVINCIAL_PASS_TO_PRELIMINARY_REVIEW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PRE_APPLYING.getCode());
    }

    @OnTransition(source = "JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS", target = "THE_JOINT_REVIEW_OF_PROVINCIAL_DEPARTMENTS_FAILED")
    public void PROVINCIAL_DEPARTMENT_REVIEW_REJECT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.THE_JOINT_REVIEW_OF_PROVINCIAL_DEPARTMENTS_FAILED.getCode());
    }

    @OnTransition(source = "JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS", target = "PENDING_PREQUALIFICATION")
    public void JOINT_REVIEW_BY_PROVINCIAL_DEPARTMENTS_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PENDING_PREQUALIFICATION.getCode());
    }

    @OnTransition(source = "PRE_APPLYING", target = "DEPARTMENT_JOINT_REVIEW")
    public void PRELIMINARY_REVIEW_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.DEPARTMENT_JOINT_REVIEW.getCode());
    }

    @OnTransition(source = "PRE_APPLYING", target = "PREQUALIFICATION_FAILED")
    public void PRELIMINARY_REVIEW_REJECT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PREQUALIFICATION_FAILED.getCode());
    }

    @OnTransition(source = "PREQUALIFICATION_FAILED", target = "PENDING_PREQUALIFICATION_CHOICE")
    public void PRELIMINARY_REVIEW_REJECT_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PENDING_PREQUALIFICATION_CHOICE.getCode());
    }

    @OnTransition(source = "PRE_APPLYING", target = "PREQUALIFICATION_WITHDRAW_CHOICE")
    public void PRE_APPLYING_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PREQUALIFICATION_WITHDRAW_CHOICE.getCode());
    }


    @OnTransition(source = "DEPARTMENT_JOINT_REVIEW", target = "IN_THE_ANNUAL_PLAN")
    public void DEPARTMENT_UNITED_REVIEW_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.IN_THE_ANNUAL_PLAN.getCode());
    }

    @OnTransition(source = "DEPARTMENT_JOINT_REVIEW", target = "DEPARTMENT_JOINT_REVIEW_FAILED")
    public void DEPARTMENT_UNITED_REVIEW_REJECT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.DEPARTMENT_JOINT_REVIEW_FAILED.getCode());
    }

    @OnTransition(source = "DEPARTMENT_JOINT_REVIEW", target = "PRE_APPLYING")
    public void DEPARTMENT_JOINT_REVIEW_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PRE_APPLYING.getCode());
    }


    @OnTransition(source = "IN_THE_ANNUAL_PLAN", target = "PLAN_TO_BE_DECLARED")
    public void ANNUAL_PLAN_PROJECT_OPEN_PLAN_DECLARE(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PLAN_TO_BE_DECLARED.getCode());
    }

    @OnTransition(source = "IN_THE_ANNUAL_PLAN", target = "BE_SUSPENDED")
    public void ANNUAL_PLAN_SUSPEND(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.BE_SUSPENDED.getCode());
    }

    @OnTransition(source = "IN_THE_ANNUAL_PLAN", target = "DEPARTMENT_JOINT_REVIEW")
    public void IN_THE_ANNUAL_PLAN_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.DEPARTMENT_JOINT_REVIEW.getCode());
    }


    @OnTransition(source = "PLAN_TO_BE_DECLARED", target = "SCHEME_UNDER_REVIEW")
    public void DECLARE_PLAN(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.SCHEME_UNDER_REVIEW.getCode());
    }

    @OnTransition(source = "PLAN_TO_BE_DECLARED", target = "IN_THE_ANNUAL_PLAN")
    public void PLAN_TO_DECLARE_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.IN_THE_ANNUAL_PLAN.getCode());
    }

    @OnTransition(source = "SCHEME_REVIEW_FAILED", target = "SCHEME_UNDER_REVIEW")
    public void DECLARE_PLAN_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.SCHEME_UNDER_REVIEW.getCode());
    }

    @OnTransition(source = "SCHEME_UNDER_REVIEW", target = "TO_BE_APPROVED")
    public void PLAN_REVIEW_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.TO_BE_APPROVED.getCode());
    }

    @OnTransition(source = "SCHEME_UNDER_REVIEW", target = "SCHEME_REVIEW_FAILED")
    public void PLAN_REVIEW_REJECT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.SCHEME_REVIEW_FAILED.getCode());
    }

    @OnTransition(source = "SCHEME_REVIEW_FAILED", target = "SCHEME_UNDER_REVIEW")
    public void PLAN_REVIEW_REJECT_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.SCHEME_UNDER_REVIEW.getCode());
    }

    @OnTransition(source = "SCHEME_UNDER_REVIEW", target = "PLAN_TO_BE_DECLARED")
    public void SCHEME_UNDER_REVIEW_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.PLAN_TO_BE_DECLARED.getCode());
    }

    //==================================================================================================================

    @OnTransition(source = "DECLARED_APPROVED_TO_BE_RECORD", target = "DECLARED_APPROVED_RECORD_AUDITING")
    public void DECLARED_RECORD_SUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 待立项批复批复后，项目一级状态变更为已立项
        project.setStatus(ProjectStatusEnum.DECLARED_APPROVED_RECORD_AUDITING.getCode());
    }

    @OnTransition(source = "DECLARED_APPROVED_RECORD_FAILED", target = "DECLARED_APPROVED_RECORD_AUDITING")
    public void DECLARED_RECORD_RESUBMIT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.DECLARED_APPROVED_RECORD_AUDITING.getCode());
    }

    @OnTransition(source = "DECLARED_APPROVED_RECORD_AUDITING", target = "TO_BE_PURCHASED")
    public void DECLARED_RECORD_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 待立项批复批复后，项目一级状态变更为已立项
        project.setStage(ProjectStatusEnum.PROJECT_APPROVED.getCode());
        project.setStatus(ProjectStatusEnum.TO_BE_PURCHASED.getCode());
    }

    @OnTransition(source = "DECLARED_APPROVED_RECORD_AUDITING", target = "DECLARED_APPROVED_RECORD_FAILED")
    public void DECLARED_RECORD_FAILED(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.DECLARED_APPROVED_RECORD_FAILED.getCode());
    }

    //==================================================================================================================

    @OnTransition(source = "TO_BE_APPROVED", target = "TO_BE_PURCHASED")
    public void PROJECT_APPROVAL(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 待立项批复批复后，项目一级状态变更为已立项
        project.setStage(ProjectStatusEnum.PROJECT_APPROVED.getCode());
        project.setStatus(ProjectStatusEnum.TO_BE_PURCHASED.getCode());
    }

    @OnTransition(source = "TO_BE_APPROVED", target = "SCHEME_UNDER_REVIEW")
    public void TO_BE_APPROVED_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.SCHEME_UNDER_REVIEW.getCode());
    }

    @OnTransition(source = "OPERATION", target = "UNDER_CONSTRUCTION")
    public void START_TO_WORK(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.UNDER_CONSTRUCTION.getCode());
    }

    @OnTransition(source = "TO_BE_PURCHASED", target = "TO_BE_APPROVED")
    public void TO_BE_PURCHASED_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 待采购状态撤回后，项目一级状态回退到未立项
        project.setStage(ProjectStatusEnum.NOT_APPROVED.getCode());
        project.setStatus(ProjectStatusEnum.TO_BE_APPROVED.getCode());
    }

    @OnTransition(source = "UNDER_CONSTRUCTION", target = "TO_BE_PURCHASED")
    public void UNDER_CONSTRUCTION_WITHDRAW(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.TO_BE_PURCHASED.getCode());
    }

    @OnTransition(source = "TO_BE_PURCHASED", target = "ON_PURCHASING")
    public void SUBMIT_PURCHASE_NOTICE(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.ON_PURCHASING.getCode());
    }

    @OnTransition(source = "ON_PURCHASING", target = "TO_BE_FIRST_INSPECTED")
    public void SUBMIT_OPERATION_PLAN(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.TO_BE_FIRST_INSPECTED.getCode());
    }

    @OnTransition(source = "TO_BE_FIRST_INSPECTED", target = "ON_PILOT_RUNNING")
    public void SUBMIT_FIRST_INSPECTED_FILES(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.ON_PILOT_RUNNING.getCode());
    }

    @OnTransition(source = "ON_PILOT_RUNNING", target = "ON_FINALLY_INSPECTED")
    public void FINAL_ACCEPTANCE_APPLICATION(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.ON_FINALLY_INSPECTED.getCode());
    }

    @OnTransition(source = "ON_FINALLY_INSPECTED", target = "ARCHIVED")
    public void FINAL_ACCEPTANCE_PASS(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        // 终验审核通过后，项目一级状态变更为已归档
        project.setStage(ProjectStatusEnum.ARCHIVED.getCode());
        project.setStatus(ProjectStatusEnum.ACCEPTED.getCode());
    }

    @OnTransition(source = "ON_FINALLY_INSPECTED", target = "FINALLY_INSPECTED_FAILED")
    public void FINAL_ACCEPTANCE_REJECT(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.FINALLY_INSPECTED_FAILED.getCode());
    }

    @OnTransition(source = "FINALLY_INSPECTED_FAILED", target = "ON_FINALLY_INSPECTED")
    public void FINAL_RE_ACCEPTANCE_APPLICATION(Message<ProjectStateChangeEvent> message) {
        Project project = getProject(message);
        project.setStatus(ProjectStatusEnum.ON_FINALLY_INSPECTED.getCode());
    }

}
