package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.builder.BaseStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.AdaptStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.TenderStateChangeEvent;
import com.hz.pm.api.common.statemachine.event.XcfhxStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.TenderAdaptStatusEnum;
import com.hz.pm.api.projectlib.model.enumeration.TenderStatusEnum;
import com.hz.pm.api.projectlib.model.enumeration.TenderXcfhxApplyStatusEnum;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * <p>
 * 标段信创符合性申请状态机
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/2/24
 */
@Slf4j
@Component
@EnableStateMachine(name = XcfhxStateMachineBuilderImpl.MACHINE_ID)
@RequiredArgsConstructor
public class XcfhxStateMachineBuilderImpl implements BaseStateMachineBuilder<Purchase, XcfhxStateChangeEvent, TenderXcfhxApplyStatusEnum> {

    public static final String MACHINE_ID = "xcfhxStateMachine";

    private final BeanFactory beanFactory;

    @Override
    public StateMachine<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent> build() throws StateMachineException {
        StateMachine<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：{}", stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    @Override
    public StateMachine<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构建失败", e);
        }
    }

    private StateMachine<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent> buildStateMachine(BeanFactory factory) throws Exception {
        StateMachineBuilder.Builder<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(factory);

        builder.configureStates()
                .withStates()
                .initial(TenderXcfhxApplyStatusEnum.TO_BE_SUBMIT_XCFHX_APPLY)
                .states(EnumSet.allOf(TenderXcfhxApplyStatusEnum.class));

        builder.configureTransitions()
                // 信创符合性申请
                .withExternal()
                .source(TenderXcfhxApplyStatusEnum.TO_BE_SUBMIT_XCFHX_APPLY)
                .target(TenderXcfhxApplyStatusEnum.ON_XCFHX_APPLY)
                .event(XcfhxStateChangeEvent.SUBMIT_XCFHX_APPLY)
                .and()// 信创符合性申请
                .withExternal()
                .source(TenderXcfhxApplyStatusEnum.XCFHX_APPLY_FAILED)
                .target(TenderXcfhxApplyStatusEnum.ON_XCFHX_APPLY)
                .event(XcfhxStateChangeEvent.RESUBMIT_XCFHX_APPLY)
                .and()
                // 信创符合性申请通过
                .withExternal()
                .source(TenderXcfhxApplyStatusEnum.ON_XCFHX_APPLY)
                .target(TenderXcfhxApplyStatusEnum.XCFHX_APPLY_PASSED)
                .event(XcfhxStateChangeEvent.XCFHX_APPLY_PASSED)
                .and()
                // 信创符合性申请失败
                .withExternal()
                .source(TenderXcfhxApplyStatusEnum.ON_XCFHX_APPLY)
                .target(TenderXcfhxApplyStatusEnum.XCFHX_APPLY_FAILED)
                .event(XcfhxStateChangeEvent.XCFHX_APPLY_FAILED)
                .and();
        return builder.build();
    }

    @Override
    @Bean(name = "xcfhxStatePersister")
    public StateMachinePersister<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent, Purchase> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent, Purchase>() {
            @Override
            public void write(StateMachineContext<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent> context, Purchase contextObj) {
                log.info("当前项目为：{}", contextObj);
            }

            @Override
            public StateMachineContext<TenderXcfhxApplyStatusEnum, XcfhxStateChangeEvent> read(Purchase contextObj) {
                TenderXcfhxApplyStatusEnum status = TenderXcfhxApplyStatusEnum.get(contextObj.getXcfhxApplyStatus())
                        .orElseThrow(() -> BizException.wrap("信创符合性申请状态无效"));
                return new DefaultStateMachineContext<>(status, null, null, null, null, MACHINE_ID);
            }
        });
    }

}
