package com.hz.pm.api.dashboard.handle;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.hz.pm.api.dashboard.constant.DashboardConstant;
import com.hz.pm.api.dashboard.model.vo.PortraitProjectOutputVO;
import com.hz.pm.api.projectlib.model.entity.ProjectApplication;
import com.hz.pm.api.projectlib.service.IProjectApplicationService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

/**
 * @Classname ApplicationHandler
 * @Description
 * @Date 2023/10/9 14:11
 * @Author PoffyZhang
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class ApplicationHandler {

    private final IProjectApplicationService applicationService;

    public List<PortraitProjectOutputVO> generateIrsApp(String projectCode, Integer version) {
        //.查询应用
        Wrapper<ProjectApplication> paQuery = Wrappers.lambdaQuery(ProjectApplication.class)
                .eq(ProjectApplication::getProjectVersion, version)
                .eq(ProjectApplication::getProjectCode, projectCode);
        List<ProjectApplication> apps = applicationService.list(paQuery);
        if (apps.isEmpty()) {
            return Collections.emptyList();
        }
        List<PortraitProjectOutputVO> retApps = Lists.newArrayList();
        for (ProjectApplication app : apps) {
            String appName = StrUtil.blankToDefault(app.getApplicationName(), app.getRelatedExistsApplication());
            retApps.add(generateOutputEntity(appName));
        }
        return retApps;
    }

    private PortraitProjectOutputVO generateOutputEntity(String name) {
        return PortraitProjectOutputVO.builder()
                .name(DashboardConstant.Protrait.PROJECT_SIGNS)
                .generateData(new PortraitProjectOutputVO.GenerateData())
                .componentUsage(new PortraitProjectOutputVO.ComponentUsage())
                .cloudResourceUsage(new PortraitProjectOutputVO.CloudResourceUsage())
                .dataUsage(new PortraitProjectOutputVO.DataUsage())
                .operationReport(new PortraitProjectOutputVO.OperationReport())
                .build();
    }
}
