package com.hz.pm.api.projectlib.handle.stage;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.ProcessStatusEnum;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * 立项批复处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ProjectApprovalHandle extends AbstractProcessHandle {

    private final IProjectStatusChangeService projectStatusChangeService;
    private final IProjectService projectService;
    private final IProjectInstService projectInstService;
    private final ProcessInstanceService processInstanceService;

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_APPROVAL;
    }

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        Project project = ctx.getProject();
        Integer status = project.getStatus();
        // 项目状态为待立项批复之前的状态
        if (status == null || (status < ProjectStatusEnum.TO_BE_APPROVED.getCode()
                && status < ProjectStatusEnum.DECLARED_APPROVED_TO_BE_RECORD.getCode())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else {
            List<Long> projectIds = projectService.allVersionProjectIds(project.getProjectCode());
            // 当前项目状态为待立项批复
            if (ProjectStatusEnum.TO_BE_APPROVED.eq(status)
                    || ProjectStatusEnum.DECLARED_APPROVED_TO_BE_RECORD.eq(status)
                    || ProjectStatusEnum.DECLARED_APPROVED_RECORD_AUDITING.eq(status)) {
                // 根据建设方案评审通过的时间获取
                final List<ProjectStateChangeEvent> events = Arrays.asList(ProjectStateChangeEvent.PLAN_REVIEW_PASS,
                        ProjectStateChangeEvent.DECLARED_RECORD_SUBMIT,
                        ProjectStateChangeEvent.DECLARED_RECORD_RESUBMIT);
                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectIds, events);
                if (projectStatusChange != null) {
                    detail.setFinishTime(projectStatusChange.getCreateOn());
                }
                detail.setStepStatus(StepStatusEnum.ON_GOING);
            } else {
                // 取 状态机 改变状态时间
                final List<ProjectStateChangeEvent> events = Arrays.asList(ProjectStateChangeEvent.PLAN_REVIEW_PASS,
                        ProjectStateChangeEvent.DECLARED_RECORD_FAILED,
                        ProjectStateChangeEvent.DECLARED_RECORD_PASS);
                ProjectStatusChange projectStatusChange = projectStatusChangeService.getLastOne(projectIds, events);
                if (Objects.nonNull(projectStatusChange)) {
                    detail.setFinishTime(projectStatusChange.getCreateOn());
                    detail.setStepStatus(StepStatusEnum.COMPLETED);
                    ProjectInst inst = projectInstService.getOne(Wrappers.lambdaQuery(ProjectInst.class)
                            .in(ProjectInst::getProjectId, projectIds)
                            .eq(ProjectInst::getInstType, InstTypeEnum.DECLARED_RECORD.getCode())
                            .orderByDesc(ProjectInst::getCreateOn)
                            .last(BizConst.LIMIT_1));
                    if (inst != null) {
                        ProcessProgressVo instDetail = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
                        detail.setProcessProgressVo(instDetail);
                        if (ProcessStatusEnum.BE_REJECTED.getDesc().equals(instDetail.getStatus())) {
                            detail.setStepStatus(StepStatusEnum.REJECTED);
                        }
                    }
                }
            }
        }
        ctx.setPreStepStatus(detail.getStepStatus());
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
